/* =============================================================================
 *
 *  Description: This is a C implementation for Thread lwip_sysboot_threadtype
 *
 * -----------------------------------------------------------------------------
 *  Comments:
 *
 * ===========================================================================*/

/* Get access to any of the VDK features & datatypes used */
#include "lwip_sysboot_threadtype.h"
#include <lwip/cglobals.h>
#include <lwip/kernel_abs.h>
#include <lwip/ADI_TOOLS_IOEThreadType.h>
#include <lwip/sockets.h>
#include <lwip/inet.h>
#include <sys/exception.h>
#include <services/services.h>
#include <drivers/adi_dev.h>
#include <stdio.h>
#include <string.h>

#define ENABLE_BF

/* Size of the memory block to allocate to the stack */
#define ETHER_STACK_SIZE    1024*1024*1

#define DEBUG_PRINT(msg,result)		{									\
										if(result)						\
											{							\
												printf(msg);			\
												return -1;				\
										}								\
									}
                      
								
/* Initializes the Stack */
int system_init(void);

/*
 *  Global static data for various SSL managers 
 */
static ADI_INT_IMASK    imask_storage;
#pragma alignment_region (4)
static u8 devmgr_storage[ADI_DEV_BASE_MEMORY + (ADI_DEV_DEVICE_MEMORY * 2)];
static u8 intmgr_storage[                      (ADI_INT_SECONDARY_MEMORY * 8)];
#pragma alignment_region_end

/*
 *  ADSP-BF533 EZ-KIT Lite w/ USBLAN specific settings 
 */
 
//#if (defined(__ADSPBF533__) || defined(USB_LAN)  )
#include <lwip/ADI_ETHER_USBLAN.h>
#define NO_RCVES_LAN 60
#define NO_XMITS_LAN 60
#pragma alignment_region (4)
char BaseMemSize_LAN[ADI_ETHER_MEM_USBLAN_BASE_SIZE];
#pragma alignment_region_end
static int p_period_LAN =10;
//#endif /* __ADSPBF533__ */

/*
 *  ADSP-BF537 EZ-KIT specific settings 
 */
//#if (!defined(USB_LAN) && defined(__ADSPBF537__) )
#include <lwip/ADI_ETHER_BF537.h>
static char hwaddr_BF[6] = { 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd };
#define NO_RCVES_BF 60 
#define NO_XMITS_BF 60
#pragma alignment_region (32)
char BaseMemSize_BF[ADI_ETHER_MEM_BF537_BASE_SIZE];
char MemRcve_BF[NO_RCVES_BF*ADI_ETHER_MEM_BF537_PER_RECV];
char MemXmit_BF[NO_XMITS_BF*ADI_ETHER_MEM_BF537_PER_XMIT];
#pragma alignment_region_end
static int p_period_BF =40;
//#endif /* __ADSPBF537__ */


//#if( !defined(USB_LAN) && defined(__ADSPBF537__) )
ADI_ETHER_SUPPLY_MEM memtable_BF = {
    MemRcve_BF,sizeof(MemRcve_BF),0,
    MemXmit_BF,sizeof(MemXmit_BF),0,
    BaseMemSize_BF,sizeof(BaseMemSize_BF)};
//#endif
//#if ( defined(USB_LAN) || defined(__ADSPBF533__) )
ADI_ETHER_SUPPLY_MEM memtable_LAN = {
    0,0,0,
    0,0,0,
	BaseMemSize_LAN,sizeof(BaseMemSize_LAN)};
//#endif
    

void
lwip_sysboot_threadtype_RunFunction(void **inPtr)
{
	char  ip[32];
	/* Initializes the TCP/IP Stack and returns */
	if(system_init() == -1)
	{
		printf("Failed to initialize system\n");
		return;
	}
	
	/* start stack */
	start_stack();
	
	/*
	 * For debug purposes, printf() the IP address to the VisualDSP++
	 * console window.  Likely not needed in final application.
	 */
	 
	memset(ip,0,sizeof(ip));
	if(gethostaddr(0,ip))
	{
		printf("IP ADDRESS of interface 0: %s\n",ip);
	}
#ifdef ENABLE_BF
	if(gethostaddr(1,ip))
	{
		printf("IP ADDRESS of interface 1: %s\n",ip);
	}
#endif

		
	/**
	 *  Add Application Code here 
	 **/
	//printf("\nWebserver running# Type  \'http://%s/blackfin.html\' from window|unix browser\n",ip);

	// Start the servers 

  struct VDK_ThreadCreationBlock TCB = {
                                         (VDK_ThreadType)0,
                                         (VDK_ThreadID)0,
                                         0,
                                         (VDK_Priority)0
                                        };
    // Create application threads.
    
    TCB.template_id = kCharGen_Server_ThreadType;
    if (VDK_CreateThreadEx(&TCB) == UINT_MAX)
    {
      fprintf(stderr, "boot: failed to create CharGen server thread\n");
      exit(905);
    }
    
    TCB.template_id = kDiscard_Server_ThreadType;
    if (VDK_CreateThreadEx(&TCB) == UINT_MAX)
    {
      fprintf(stderr, "boot: failed to create Discard server thread\n");
      exit(906);
    }
    
    TCB.template_id = kEcho_Server_ThreadType;    
    if (VDK_CreateThreadEx(&TCB) == UINT_MAX)
    {
      fprintf(stderr, "boot: failed to create Echo server thread\n");
      exit(907);
    }
	
   // Start the webserver
   web_main(0,0);
   
   
   //while(1);
   
    /* Put the thread's exit from "main" HERE */
    /* A thread is automatically Destroyed when it exits its run function */    
    
}

int
lwip_sysboot_threadtype_ErrorFunction(void **inPtr)
{

    /* TODO - Put this thread's error handling code HERE */

      /* The default ErrorHandler kills the thread */

	VDK_DestroyThread(VDK_GetThreadID(), false);
	return 0;
}

void
lwip_sysboot_threadtype_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
{
    /* Put code to be executed when this thread has just been created HERE */

    /* This routine does NOT run in new thread's context.  Any non-static thread
     *   initialization should be performed at the beginning of "Run()."
     */
}

void
lwip_sysboot_threadtype_DestroyFunction(void **inPtr)
{
    /* Put code to be executed when this thread is destroyed HERE */

    /* This routine does NOT run in the thread's context.  Any VDK API calls
     *   should be performed at the end of "Run()."
     */
}

/******************************************************************
 * 
 *  Function: system_init()
 *  Description:
 *  		Initializes Device Manager, Interrupt
 *  		Manager and the Stack. 
 *   Return:
 *   		Returns 1 upon success -1 upon failure.
 *
 ******************************************************************/
int system_init()
{
	ADI_DEV_MANAGER_HANDLE devmgr_handle;
	unsigned int result;
	u32 response_count,critical_reg;
	
	ADI_DEV_DEVICE_HANDLE handles[2];
	#define	lan_handle ( handles[0] )
	#define bf_handle ( handles[1] )

	char *ether_stack_block;
	u16 phyregs[32];
//#if ( !defined(USB_LAN)  && defined(__ADSPBF537__) )
	const unsigned int mac_address_in_flash_BF = 0x203F0000;
//#endif

  
	/* initialize the interrupt manager */

	result = adi_int_Init(
                           intmgr_storage,
                           sizeof(intmgr_storage),
                           &response_count,
                           &critical_reg);
	 
	DEBUG_PRINT("Failed to Initialize interrupt manager\n",result != ADI_DEV_RESULT_SUCCESS);

	/* initialize the device manager */

	result = adi_dev_Init(
                            devmgr_storage,
                            sizeof(devmgr_storage),
                            &response_count,
                            &devmgr_handle,
                            &imask_storage);
	
	DEBUG_PRINT("Failed to Initilize device manager\n",result != ADI_DEV_RESULT_SUCCESS);
  
	/* Initialize the kernel */

	ker_init((void*)0);
  
	/* set thread type for the stack threads */

	ker_set_auxdata((void*)kADI_TOOLS_IOEThreadType);


	
	/* open lan-device */

	result = adi_dev_Open(
                           devmgr_handle,
                           &ADI_ETHER_USBLAN_Entrypoint,
                           0,
                           NULL,
                           &lan_handle,
                           ADI_DEV_DIRECTION_BIDIRECTIONAL,
                           NULL,
                           NULL,
                           (ADI_DCB_CALLBACK_FN)stack_callback_handler);
	DEBUG_PRINT("Failed to open the lan-device\n",result != ADI_DEV_RESULT_SUCCESS);
                           
#ifdef ENABLE_BF                       
    result = adi_dev_Open(
                           devmgr_handle,
                           &ADI_ETHER_BF537_Entrypoint,
                           0,
                           NULL,
                           &bf_handle,
                           ADI_DEV_DIRECTION_BIDIRECTIONAL,
                           NULL,
                           NULL,
                           (ADI_DCB_CALLBACK_FN)stack_callback_handler);

	DEBUG_PRINT("Failed to open the bf-device\n",result != ADI_DEV_RESULT_SUCCESS);
#endif
 
	/* set the services with in stack */
#ifdef ENABLE_BF
	set_pli_services( 2,handles );
#else
	set_pli_services( 1,handles );
#endif


//#if ( !defined(USB_LAN) && defined(__ADSPBF537__) )
	/*
	 * Read the EZ-KIT Lite's assigned MAC address, found at address 0x203F0000.
	 * We need to first set the AMGCTL register to allow access to asynchronous
	 * memory. 
	 * 
	 * DMA gets more priority than the processor while accessing SDRAM.
	 */
#ifdef ENABLE_BF
	*pEBIU_AMGCTL = 0xFF | 0x100;
	memcpy ( &hwaddr_BF, (unsigned char *) mac_address_in_flash_BF, sizeof ( hwaddr_BF ) );	
	result = adi_dev_Control(
                              bf_handle,
                              ADI_ETHER_CMD_SET_MAC_ADDR,
                              (void*)&hwaddr_BF);                              
	DEBUG_PRINT("Failed set MAC address for BF\n",result != ADI_DEV_RESULT_SUCCESS);
#endif
//#endif 
	
	/* supply some memory for the driver */

	result = adi_dev_Control(
                              lan_handle,
                              ADI_ETHER_CMD_SUPPLY_MEM,
                              &memtable_LAN);

	DEBUG_PRINT("Failed to supply memory to LAN driver\n",result != ADI_DEV_RESULT_SUCCESS);

#ifdef ENABLE_BF
	result = adi_dev_Control(
                              bf_handle,
                              ADI_ETHER_CMD_SUPPLY_MEM,
                              &memtable_BF);

	DEBUG_PRINT("Failed to supply memory to BF driver\n",result != ADI_DEV_RESULT_SUCCESS);

#endif

	result = adi_dev_Control(
                              lan_handle,
                              ADI_DEV_CMD_SET_DATAFLOW_METHOD,
                              (void*)TRUE);
	DEBUG_PRINT("Failed to set DATAFLOW to LAN driver\n",result != ADI_DEV_RESULT_SUCCESS);                            

#ifdef ENABLE_BF	
    result = adi_dev_Control(
                              bf_handle,
                              ADI_DEV_CMD_SET_DATAFLOW_METHOD,
                              (void*)TRUE);
	DEBUG_PRINT("Failed to set DATAFLOW to bf driver\n",result != ADI_DEV_RESULT_SUCCESS);                            
#endif

	/* Initialze the stack with user specified configuration priority -3 and
	 * poll period of p_period msec.  The stack is allocated a memory buffer as well.
	 */
   
	ether_stack_block = (char *) malloc ( ETHER_STACK_SIZE );

	DEBUG_PRINT("Failed to malloc stack \n",!ether_stack_block);
   
	init_stack ( 3, p_period_LAN, ETHER_STACK_SIZE, ether_stack_block ); // p_period_LAN != p_period_BF though
	
	/* Start the MAC */
	result = adi_dev_Control (
								lan_handle,
								ADI_ETHER_CMD_START,
								NULL);
	DEBUG_PRINT("Failed to start the LAN driver\n",result != ADI_DEV_RESULT_SUCCESS);
	
#ifdef ENABLE_BF
	result = adi_dev_Control (
								bf_handle,
								ADI_ETHER_CMD_START,
								NULL);
	DEBUG_PRINT("Failed to start the BF driver\n",result != ADI_DEV_RESULT_SUCCESS);
#endif		
	
	/* read the LAN controller registers */
	adi_dev_Control(lan_handle,ADI_ETHER_CMD_GET_PHY_REGS,phyregs);	
	DEBUG_PRINT("LAN Controller has failed and the board needs power cycled\n",phyregs[1]==0xFFFF);
	
	/* wait for the link to be up */
	if ( (phyregs[1]&0x4) ==0) // check link status of the phy
	{
		printf("Waiting for the LAN link to be established\n");
		while ( (phyregs[1]&0x4) ==0)
		{		
			adi_dev_Control(lan_handle,ADI_ETHER_CMD_GET_PHY_REGS,phyregs);
		}
	}	
	printf("LAN link established\n");

#ifdef ENABLE_BF
	/* read the BF controller register */
	adi_dev_Control(bf_handle,ADI_ETHER_CMD_GET_PHY_REGS,phyregs);	
	DEBUG_PRINT("BF Controller has failed and the board needs power cycled\n",phyregs[1]==0xFFFF);
	
	/* wait for the link to be up */
	if ( (phyregs[1]&0x4) ==0) // check link status of the phy
	{
		printf("Waiting for the BF link to be established\n");
		while ( (phyregs[1]&0x4) ==0)
		{		
			adi_dev_Control(bf_handle,ADI_ETHER_CMD_GET_PHY_REGS,phyregs);
		}
	}	
	printf("BF Link established\n");
#endif	
    
	return 1;
}
/* ========================================================================== */
